@@ -1,10 +1,10 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "org-repeat-by-cron" "1.1.6"
+(define-package "org-repeat-by-cron" "20250922.610"
   "An Org mode task repeater based on Cron expressions."
   '((emacs "24.4"))
   :url "https://github.com/TomoeMami/org-repeat-by-cron.el"
-  :commit "9a382e817dc63d5f8a6c74bd9d9233a14f1f3c96"
-  :revdesc "9a382e817dc6"
+  :commit "1bd31d7aa0b372703db0b32c7e4c9fd99b54dc34"
+  :revdesc "1bd31d7aa0b3"
   :keywords '("calendar")
   :authors '(("TomoeMami" . "trembleafterme@outlook.com"))
   :maintainers '(("TomoeMami" . "trembleafterme@outlook.com")))
