@@ -8,8 +8,8 @@
 ;; Keywords: calendar
 ;; URL: https://github.com/TomoeMami/org-repeat-by-cron.el
 
-;; Package-Version: 1.1.6
-;; Package-Revision: 9a382e817dc6
+;; Package-Version: 20250922.610
+;; Package-Revision: 1bd31d7aa0b3
 ;; Package-Requires: ((emacs "24.4"))
 
 ;; This file is not part of GNU Emacs.
@@ -29,178 +29,114 @@
 
 ;;; Commentary:
 ;; An Org mode task repeater based on Cron expressions
-;; 
 ;; Modified from https://github.com/Raemi/org-reschedule-by-rule.
-;; 
 ;; Key Differences:
 ;; - Uses a cron parser implemented in pure Elisp, with
 ;;   no dependency on the Python croniter package.
 ;; - Replaces the INTERVAL property with a DAY_AND property.
 ;; - Supports toggling between SCHEDULED and DEADLINE timestamps.
-;; - Fully compatible with org-habit.
 ;; 
 ;; org-repeat-by-cron.el is a lightweight extension for Emacs Org
-;; mode that allows you to repeat tasks using the power of Cron expressions.
-;; Standard Org mode repeaters (like +1d , ++1w ) are relative to
+  ;; mode that allows you to repeat tasks based on powerful Cron expressions.
+;; Standard Org mode repeaters (like +1d , ++1w ) are based on
 ;; the current SCHEDULED or DEADLINE timestamp. In contrast, this
-;; tool provides repetition based on absolute time rules. You can
-;; easily set a task to repeat "on the last Friday of every month"
-;; or "on the first Monday of each quarter" without manual date
-;; calculations.
-;; 
-;; A core advantage is its pure Elisp implementation, ensuring it
-;; works out-of-the-box in any Emacs environment without external
-;; dependencies.
+;; tool provides a repetition method based on absolute time rules.
+;; You can easily set a task to repeat \"on the last Friday of
+;; every month\" or \"on the first Monday of each quarter\"
+;; without manual date calculations.
+;; A core advantage of this tool is its pure Elisp implementation,
+;; which does not rely on any external programs (like Python's
+;; croniter library), ensuring it works out-of-the-box in Emacs environment.
 ;; 
 ;; Installation
-;; Install via Melpa :
+;; - Download =org-repeat-by-cron.el= and place it in your Emacs load-path.
+;; - Add the following code to your Emacs configuration file (e.g., init.el ):
+;;   #+begin_src elisp
+;; (require 'org-repeat-by-cron)
+;;   #+end_src
+;; 
+;; Using use-package
+;; 
+;; If you use use-package , the configuration is more concise:
 ;; #+begin_src elisp
 ;; (use-package org-repeat-by-cron
-;;   :ensure t
-;;   :config
-;;   (global-org-repeat-by-cron-mode))
+;;   :ensure nil  ; If the file is already in your load-path
+;;   :load-path "/path/to/your/lisp/directory/")
 ;; #+end_src
 ;; 
-;; Usage
-;; 
-;; To make an Org task repeat according to a Cron rule,
-;; simply add the =REPEAT_CRON= property to its =PROPERTIES= drawer.
-;;
-;; Tip: You do not need to wrap the =REPEAT_CRON= value in quotes.
-;; 
-;; Example
-;; 
-;; Suppose we have a weekly course:
-;; 
-;; #+begin_src org
-;; ,* TODO Weekend Course
-;; :PROPERTIES:
-;; :REPEAT_CRON: * * SAT,SUN
-;; :END:
-;; #+end_src
-;; 
-;; When marked as =DONE=, it will automatically be rescheduled
-;; to the next matching date. For example, if today is Dec 9, 2025,
-;; it will be scheduled for the coming Saturday (Dec 13, 2025):
-;; 
-;; #+begin_src org
-;; ,* TODO Weekend Course
-;; SCHEDULED: <2025-12-13 Sat>
-;; :PROPERTIES:
-;; :REPEAT_CRON: * * SAT,SUN
-;; :REPEAT_ANCHOR: 2025-12-13 Sat
-;; :END:
-;; #+end_src
-;; 
-;; Marking it =DONE= again will calculate the next point based
-;; on the =REPEAT_ANCHOR= and the current time:
-;; 
-;; #+begin_src org
-;; ,* TODO Weekend Course
-;; SCHEDULED: <2025-12-14 Sun>
-;; :PROPERTIES:
-;; :REPEAT_CRON: * * SAT,SUN
-;; :REPEAT_ANCHOR: 2025-12-14 Sun
-;; :END:
-;; #+end_src
-;; 
-;; Tip1: If you do not want to specify a specific hour/minute,
-;; use the *3-field shorthand* (discussed below).
-;; Tip2: If you use org-repeat-by-cron with the built-in Org repeater
-;; cookie (e.g., +1w) on the same task, the build-in cookie will
-;; be taken over and preserved, so you can use this package with
-;; org-habit.
-;;
-;; Check README for more usage: https://github.com/TomoeMami/org-repeat-by-cron.el
-;; 
+;; Tip: You should not use org-repeat-by-cron and the built-in
+;; Org repeater cookie (e.g., +1w) on the same task.
+
 ;;; Code:
 
 (require 'cl-lib)
 (require 'org)
 
 (defgroup org-repeat-by-cron nil
-  "Cron-style task repetition for Org mode.
-
-This group contains options for a system that allows Org tasks to repeat
-according to complex cron-style schedules instead of simple intervals."
+  "Customizations for org-repeat-by-cron-mode."
   :group 'org)
 
 (defconst org-repeat-by-cron--month-aliases
   '(("jan" . "1") ("feb" . "2") ("mar" . "3") ("apr" . "4") ("may" . "5")
     ("jun" . "6") ("jul" . "7") ("aug" . "8") ("sep" . "9") ("oct" . "10")
     ("nov" . "11") ("dec" . "12"))
-  "Alist mapping month name abbreviations to their numeric strings.")
+  "Mapping of month name abbreviations to numbers.")
 
 (defconst org-repeat-by-cron--dow-aliases
   '(("sun" . "0") ("mon" . "1") ("tue" . "2") ("wed" . "3") ("thu" . "4")
     ("fri" . "5") ("sat" . "6"))
-  "Alist mapping day-of-week abbreviations to their numeric strings.")
+  "Mapping of day-of-week name abbreviations to numbers (0=Sunday).")
 
 (defcustom org-repeat-by-cron-anchor-prop "REPEAT_ANCHOR"
-  "Org property name used to store the last calculated repeat timestamp.
+  "Name of the Org property for the cron repetition anchor timestamp.
 
-This property acts as the reference point for calculating the next
-repetition occurrence to ensure consistency when a task is completed."
+This variable holds the string used as the property key to store
+the base timestamp for cron-based repetitions.  When a repeating
+entry is marked as done, the function `org-repeat-by-cron-on-done'
+looks for this property to determine the starting point for
+calculating the next occurrence.  The property's value is then
+updated to the new scheduled time."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom org-repeat-by-cron-day-and-prop "REPEAT_DAY_AND"
-  "Org property name that determines the logic for day matching.
+  "Name of the Org property to control the day-of-month/day-of-week logic.
 
-If this property is set to \"t\" in an entry, the next occurrence
-must satisfy both the day-of-month and the day-of-week rules.
-Otherwise, it satisfies either rule (standard cron behavior)."
+This variable holds the string used as the property key to specify
+how the day-of-month and day-of-week fields of a cron rule are
+combined.  The function `org-repeat-by-cron-on-done' checks for
+this property in the Org entry.
+
+If this property's value is set to the string \"t\", a date must
+satisfy both the day-of-month and day-of-week rules (logical AND).
+Otherwise, a date matches if it satisfies either rule (logical OR)."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom  org-repeat-by-cron-cron-prop "REPEAT_CRON"
-  "Org property name used to store the cron expression for a task.
+  "Name of the Org property for the cron repetition rule.
 
-The value should be a standard 5-field cron string or a 3-field
-string (Day Month Day-of-Week) for daily-level precision."
+This variable holds the string used as the property key to store
+the cron repetition rule itself.  The function
+`org-repeat-by-cron-on-done' looks for an Org entry property
+with this key.  The value associated with this property must be a
+valid 3- or 5-field cron string."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom org-repeat-by-cron-deadline-prop "REPEAT_DEADLINE"
-  "Org property name defining how deadlines are rescheduled via cron.
+  "Name of the Org property to trigger deadline-based repetition.
 
-In an Org entry, the property named by this variable determines
-the logic used to move the deadline forward when a task is
-marked DONE.
-
-If the property named by this variable is set to \"t\",
-the next deadline is calculated using the primary cron expression
-from `org-repeat-by-cron-cron-prop'. This calculation uses the
-entry's current deadline as the anchor time instead of its
-scheduled time. Additionally, the existing `SCHEDULED' timestamp is
-cleared when the task is marked DONE.
-
-Setting the property value to a specific cron expression (e.g.,
-\"0 9 1 * *\") decouples the deadline from the schedule. The
-deadline will be recalculated according to its own rule,
-regardless of when the task is scheduled to start."
+This variable holds the string used as a property key.  When the
+function `org-repeat-by-cron-on-done' processes a completed entry,
+it checks for a property with this key.  If the property exists
+and its value is the string \"t\", the function will update the
+entry's `DEADLINE` timestamp to the next calculated time.  In
+this case, any existing `SCHEDULED` timestamp on the entry will be
+removed."
   :group 'org-repeat-by-cron
   :type 'string)
 
-(defcustom org-repeat-by-cron-deadline-anchor-prop "REPEAT_DEADLINE_ANCHOR"
-  "Org property name for storing the last calculated deadline time.
-
-This property is automatically created and used only when the property
-specified by `org-repeat-by-cron-deadline-prop' contains a specific
-cron expression. If that property is set to the string \"t\", the
-deadline is recalculated using the primary cron rule instead, and
-this anchor property will not be created or updated."
-  :group 'org-repeat-by-cron
-  :type 'string)
-
-(defcustom org-repeat-by-cron-max-search-year 50
-  "Maximum number of years to look ahead for a matching date.
-
-This prevents infinite loops if a cron expression describes a date
-that is impossible or extremely far in the future."
-  :group 'org-repeat-by-cron
-  :type 'integer)
-
 ;; --- [Date Calculation Helper Functions] ---
 
 (defun org-repeat-by-cron--substitute-aliases (field-str alias-map)
@@ -224,14 +160,11 @@ the transformed string."
                                                number-str result))))))
 
 (defun org-repeat-by-cron--get-dow (day month year)
-  "Return the day of the week for DAY, MONTH, and YEAR.
+  "Return the day of the week for the date.
 
-The result is an integer between 0 and 6, where 0 represents
-Sunday, 1 represents Monday, and so on.
-
-The arguments DAY, MONTH, and YEAR must be integers. MONTH
-should be in the range 1--12."
-  (calendar-day-of-week (list month day year)))
+Specified by DAY, MONTH, and YEAR.  The returned value is an
+integer from 0,7 (Sunday) to 6 (Saturday)."
+  (nth 6 (decode-time (encode-time 0 0 0 day month year))))
 
 (defun org-repeat-by-cron--find-nth-dow-of-month (n target-dow month year)
   "Return the day of the month for the Nth TARGET-DOW in MONTH of YEAR.
@@ -339,31 +272,25 @@ function will parse and include the numeric value (15) in the
 list, but it does not implement the logic for finding the
 nearest weekday.  That responsibility is left to the caller."
   ;; * Means all range.
-  (if (string= field-str "*")
-      (number-sequence min max)
+  (if (string= field-str "*") (number-sequence min max)
     ;; Else, prepare numbers under rules.
-    (let (values)
+    (let ((values '()))
       ;; Split by ','
       (dolist (part (split-string field-str ","))
         (cond
          ;; Step
-         ((string-match "\\(.*\\)/\\([0-9]+\\)" part)
-          (let* ((range-str (match-string 1 part))
-                 (step (string-to-number (match-string 2 part)))
-                 (range (if (string= range-str "*")
-                            (cons min max)
-                          (let ((r (split-string range-str "-")))
-                            (cons (string-to-number (car r)) (string-to-number (cadr r)))))))
-            (cl-loop for i from (car range) to (cdr range) by step
-                     do (push i values))))
+         ((string-match "/\\([0-9]+\\)" part)
+          (let* ((step (string-to-number (match-string 1 part)))
+                 (base-str (substring part 0 (match-beginning 0)))
+                 (base-range (if (string= base-str "*") (number-sequence min max)
+                               (let ((r (mapcar #'string-to-number (split-string base-str "-")))) (number-sequence (car r) (cadr r)))))
+                 (result '()))
+            (dolist (i base-range (setq result (nreverse result))) (when (zerop (% (- i min) step)) (push i result)))
+            (setq values (append result values))))
          ;; Range
-         ((string-match "\\([0-9]+\\)-\\([0-9]+\\)" part)
-          (cl-loop for i from (string-to-number (match-string 1 part))
-                   to (string-to-number (match-string 2 part))
-                   do (push i values)))
+         ((string-match "-" part) (setq values (append (let ((r (mapcar #'string-to-number (split-string part "-")))) (number-sequence (car r) (cadr r))) values)))
          ;; Ignore [0-9]+W , deal with it later
-         ((string-match "\\([0-9]+\\)W" part)
-          (push (string-to-number (match-string 1 part)) values))
+         ((string-match "\\([0-9]+\\)W" part) (push (string-to-number (match-string 1 part)) values))
          ;; Default numbers, save it directly
          (t (push (string-to-number part) values))))
       (sort (cl-delete-duplicates values) '<))))
@@ -388,7 +315,7 @@ The DOM-RULE supports several special formats:
 
 The DOW-RULE also supports special formats, which can be part of
 a comma-separated list:
-- \"5L\": The last Friday of the month.  Both 0 and 7 represent Sunday.
+- \"L5\": The last Friday of the month.  Both 0 and 7 represent Sunday.
 - \"5#3\": The third Friday of the month."
   (let ((dow (org-repeat-by-cron--get-dow day month year)) dom-match-p dow-match-p)
     ;; 1. Evaluate Day-of-Month (DOM) rule
@@ -411,8 +338,8 @@ a comma-separated list:
           (cl-some
            (lambda (part)
              (cond
-              ;; Format: 5L or 7L -> last Friday/Sunday
-              ((string-match "\\([0-7]\\)L" part)
+              ;; Format: L5 or L7 -> last Friday/Sunday
+              ((string-match "L\\([0-7]\\)" part)
                (let* ((target-dow (string-to-number (match-string 1 part)))
                       ;; If it's 7, convert to 0 (Sunday)
                       (final-dow (if (= target-dow 7) 0 target-dow)))
@@ -446,7 +373,7 @@ a comma-separated list:
 The function parses the five-field CRON-STRING to find the next valid
 occurrence, starting the search from the minute immediately after
 START-TIME.  It returns the result as an Emacs time value, or nil
-if no match is found within `org-repeat-by-cron-max-search-year'.
+if no match is found before the year 2099.
 
 CRON-STRING must contain five space-separated fields:
 1. Minute (0-59)
@@ -461,14 +388,13 @@ with comma \",\"). If DAY-AND is nil (by default), a date need only
 satisfy one of the conditions (OR). If it is t, a date must
 satisfy both conditions (AND)."
   (let* ((cron-parts (split-string cron-string "[ \t]+" t))
-         (minute-list   (org-repeat-by-cron--expand-field (nth 0 cron-parts) 0 59))
-         (hour-list  (org-repeat-by-cron--expand-field (nth 1 cron-parts) 0 23))
+         (minute-rule (nth 0 cron-parts))
+         (hour-rule (nth 1 cron-parts))
          (dom-rule (nth 2 cron-parts))
          ;; Convert abbreviations to numbers
          (month-rule (org-repeat-by-cron--substitute-aliases (nth 3 cron-parts) org-repeat-by-cron--month-aliases))
-         (month-list (org-repeat-by-cron--expand-field month-rule 1 12))
          (dow-rule (org-repeat-by-cron--substitute-aliases (nth 4 cron-parts) org-repeat-by-cron--dow-aliases))
-         (end-time-year (+ (nth 5 (decode-time)) org-repeat-by-cron-max-search-year))
+         (end-time-year 2099)
          ;; Start searching from the next minute
          (time (time-add start-time 60))
          (decoded-time (decode-time time)))
@@ -483,7 +409,7 @@ satisfy both conditions (AND)."
         ;; Iterate by year
         (cl-loop for year from start-year to end-time-year do
                  ;; Iterate through each month that matches the condition
-                 (dolist (month month-list)
+                 (dolist (month (org-repeat-by-cron--expand-field month-rule 1 12))
                    ;; If this month is greater than or equal to the starting month
                    (when (>= month (if (= year start-year) start-mon 1))
                      ;; Iterate through days of the month, starting from start day if it's the starting month
@@ -493,12 +419,12 @@ satisfy both conditions (AND)."
                               (when (org-repeat-by-cron--day-fields-match-p dom-rule dow-rule day month year day-and)
                                 (let ((current-hour-start (if (and (= year start-year) (= month start-mon) (= day start-day))
                                                               start-hour 0)))
-                                  (dolist (hour hour-list)
+                                  (dolist (hour (org-repeat-by-cron--expand-field hour-rule 0 23))
                                     ;; When hour satisfies the rule, check minutes
                                     (when (>= hour current-hour-start)
                                       (let ((current-min-start (if (and (= year start-year) (= month start-mon) (= day start-day) (= hour start-hour))
                                                                    start-min 0)))
-                                        (dolist (minute minute-list)
+                                        (dolist (minute (org-repeat-by-cron--expand-field minute-rule 0 59))
                                           (when (>= minute current-min-start)
                                             (cl-return-from finder
                                               (encode-time 0 minute hour day month year)))))))))))))
@@ -510,12 +436,11 @@ satisfy both conditions (AND)."
 The function returns 3 if RULE contains three space-separated fields,
 and 5 if it contains five.  In all other cases, it returns nil.
 Whitespace at the beginning or end of RULE is ignored."
-  (if rule
-      (let* ((parts (split-string (string-trim rule) "[ \t]+" t))
-             (n (length parts)))
-        (cond ((= n 3) 3)
-              ((= n 5) 5)
-              (t nil)))))
+  (let* ((parts (split-string (string-trim rule) "[ \t]+" t))
+         (n (length parts)))
+    (cond ((= n 3) 3)
+          ((= n 5) 5)
+          (t nil))))
 
 (defun org-repeat-by-cron--normalize-cron-rule (rule)
   "Normalize the cron RULE string to a 5-field format.
@@ -532,7 +457,7 @@ error message and returns nil."
       (3 (concat "0 0 " (string-trim rule)))
       (5 (string-trim rule))
       (_ (progn
-           (message "[Cron-Repeat] Invalid cron rule: %s" rule)
+           (message "org-repeat-by-cron: invalid cron rule: %S" rule)
            nil)))))
 
 (defun org-repeat-by-cron--reschedule-use-time-p (anchor-str cron-arity scheduled-str)
@@ -543,216 +468,103 @@ time component based on ANCHOR-STR, CRON-ARITY, and SCHEDULED-STR.
 
 A format string including time (\"%Y-%m-%d %a %H:%M\") is returned
 if any of the following conditions are met, in order of precedence:
-1.  CRON-ARITY is 5, indicating the cron rule specifies a time.
-2.  ANCHOR-STR is a non-nil string containing a time (e.g., \"HH:MM\").
+1.  ANCHOR-STR is a non-nil string containing a time (e.g., \"HH:MM\").
+2.  CRON-ARITY is 5, indicating the cron rule specifies a time.
 3.  SCHEDULED-STR is a non-nil string containing a time.
 
 If none of these conditions are true, the function returns a
 date-only format string (\"%Y-%m-%d %a\")."
-  (let ((time-regexp "[0-9]\\{2\\}:[0-9]\\{2\\}"))
-    (if (or (eq cron-arity 5)
-            (and anchor-str (string-match-p time-regexp anchor-str))
-            (and scheduled-str (string-match-p time-regexp scheduled-str)))
-        "%Y-%m-%d %a %H:%M"
-      "%Y-%m-%d %a")))
+  (let ((fmt-time  "%Y-%m-%d %a %H:%M")
+        (fmt-date  "%Y-%m-%d %a")
+        (time-regexp "[0-9]\\{2\\}:[0-9]\\{2\\}"))
+    (cond
+     (anchor-str
+      (if (string-match-p time-regexp anchor-str)
+          fmt-time
+        fmt-date))
+     ((eq cron-arity 5)
+      fmt-time)
+     ((eq cron-arity 3)
+      fmt-date)
+     (scheduled-str
+      (if (string-match-p time-regexp scheduled-str)
+          fmt-time
+        fmt-date))
+     (t fmt-date))))
 
-(defun org-repeat-by-cron--extract-repeater (ts-str)
-  "Extract the Org repeater cookie from timestamp string TS-STR.
+(defun org-repeat-by-cron-on-done ()
+  "Reschedule an Org entry using a cron rule when it is marked DONE.
 
-Return the match (e.g., \"+1d\", \"++2w\", or \".1m/3d\") if found,
-otherwise nil."
-  (when (and ts-str
-             (string-match org-repeat-re ts-str))
-    (match-string 1 ts-str)))
+This function is intended to be called from the hook
+`org-after-todo-state-change-hook'.  When an entry is marked with
+a DONE state, this function calculates the next scheduled time
+based on a cron rule and updates the entry accordingly.
 
-(defvar org-repeat-by-cron--repeater-raw nil
-  "Symbol indicating which timestamps contain native Org repeater cookies.
+The behavior is controlled by properties within the entry's drawer:
+- The property stored in `org-repeat-by-cron-cron-prop' must contain
+  the cron string.  This is required.
+- The property in `org-repeat-by-cron-anchor-prop' stores the
+  base time for calculating the next occurrence.  This function
+  automatically updates this property to the new time after a
+  reschedule.
+- If the property in `org-repeat-by-cron-deadline-prop' is \"t\",
+  this function reschedules the `DEADLINE` timestamp and clears
+  the SCHEDULED one.  Otherwise, it reschedules SCHEDULED.
+- If the property in `org-repeat-by-cron-day-and-prop' is \"t\",
+  the day-of-month and day-of-week rules are combined with a
+  logical AND; otherwise, they are combined with OR.
 
-This variable tracks whether SCHEDULED or DEADLINE timestamps
-of the current entry had a native repeater (e.g., \"+1d\") before
-cron rescheduling. It is used to determine if existing cookies
-should be preserved to maintain compatibility with `org-habit'.
-Possible values are schedule , deadline, both, or nil.")
-
-(defun org-repeat-by-cron--ensure-repeater ()
-  "Ensure the current Org entry has a repeater cookie for cron repetition.
-
-This function is called by `org-after-todo-state-change-hook' to ensure that
-entries using `org-repeat-by-cron' are recognized as repeating tasks.
-It adds a temporary repeater cookie if none is present, ensuring
-`org-trigger-hook' is run and `org-habit' consistency is maintained."
-  (save-excursion
-    (org-back-to-heading t)
-    (let* ((pom (point))
-           (cron-str (org-entry-get pom org-repeat-by-cron-cron-prop))
-           (cron-arity (org-repeat-by-cron--cron-rule-arity cron-str)))
-      (when (and cron-str (not (string-empty-p (string-trim cron-str)))
-                 ;; org-state dynamically bound in org.el/org-todo
-                 (member org-state org-done-keywords))
-        (if (not cron-arity)
-            (message "[Cron-Repeat] Invalid cron rule: %s" cron-str)
-          (let* ((deadline-prop (org-entry-get pom org-repeat-by-cron-deadline-prop))
-                 (process-deadline nil)
-                 (process-schedule nil))
-
-            (cond
-             ((string= deadline-prop "t")
-              (setq process-deadline t))
-             ((org-repeat-by-cron--cron-rule-arity deadline-prop)
-              (setq process-deadline t)
-              (setq process-schedule t))
-             (t
-              (setq process-schedule t)))
-            
-            (let ((sched-has-rep nil)
-                  (dead-has-rep nil))
-              (when process-schedule
-                (let* ((ts-str (org-entry-get pom "SCHEDULED"))
-                       (has-ts (and ts-str (not (string-empty-p ts-str))))
-                       (repeater (and has-ts (org-repeat-by-cron--extract-repeater ts-str))))
-                  (cond
-                   (repeater
-                    (setq sched-has-rep t))
-                   ;; a timestamp without repeater，add a temp repeater "+1d"
-                   (has-ts
-                    (let ((new-ts-str (if (string-match-p "\\([>]\\|\\]\\)$" ts-str)
-                                          (replace-regexp-in-string "\\([>]\\|\\]\\)$" " +1d\\1" ts-str)
-                                        (concat ts-str " +1d"))))
-                      (org-schedule nil new-ts-str)))
-                   ;; when no timestamp, create one
-                   (t
-                    (let* ((arity (org-repeat-by-cron--cron-rule-arity cron-str))
-                           (fmt (if (eq arity 5) "%Y-%m-%d %a %H:%M +1d" "%Y-%m-%d %a +1d"))
-                           (new-ts-str (format-time-string fmt (current-time))))
-                      (org-entry-put pom "SCHEDULED" (concat "<" new-ts-str ">")))))))
-            (when process-deadline
-              (let* ((ts-str (org-entry-get pom "DEADLINE"))
-                     (has-ts (and ts-str (not (string-empty-p ts-str))))
-                     (repeater (and has-ts (org-repeat-by-cron--extract-repeater ts-str)))
-                     (cron-val (if (string= deadline-prop "t") cron-str deadline-prop)))
-                (cond
-                 (repeater
-                  (setq dead-has-rep t))
-                 ;; a timestamp without repeater，add a temp repeater "+1d"
-                 (has-ts
-                  (let ((new-ts-str (if (string-match-p "\\([>]\\|\\]\\)$" ts-str)
-                                        (replace-regexp-in-string "\\([>]\\|\\]\\)$" " +1d\\1" ts-str)
-                                      (concat ts-str " +1d"))))
-                    (org-deadline nil new-ts-str)))
-                 ;; when no timestamp, create one
-                 (t
-                  (let* ((arity (org-repeat-by-cron--cron-rule-arity cron-val))
-                         (fmt (if (eq arity 5) "%Y-%m-%d %a %H:%M +1d" "%Y-%m-%d %a +1d"))
-                         (new-ts-str (format-time-string fmt (current-time))))
-                    (org-entry-put pom "DEADLINE" (concat "<" new-ts-str ">")))))))
-            
-            ;; 4. set org-repeat-by-cron--repeater-raw
-            (setq org-repeat-by-cron--repeater-raw
-                  (cond
-                   ((and sched-has-rep dead-has-rep) 'both)
-                   (sched-has-rep 'schedule)
-                   (dead-has-rep 'deadline)
-                   (t nil))))))))))
-
-
-(defun org-repeat-by-cron-on-done (change-plist)
-  "Reschedule the Org task at point according to CHANGE-PLIST and cron rules.
-
-This function is intended for `org-trigger-hook'. It reads the cron rule from
- the property named by the variable `org-repeat-by-cron-cron-prop'.
-It then calculates the next occurrence after the current time or anchor time
-and updates the SCHEDULED or DEADLINE timestamps. The anchor property
-is also updated to ensure consistent calculation for the next repetition."
-  (let* ((from-str (format "%s" (plist-get change-plist :from)))
-         (to-str (format "%s" (plist-get change-plist :to)))
-         (pom (point))
-         (cron-str (org-entry-get pom org-repeat-by-cron-cron-prop)))
-    (when (and cron-str
-               (not (string-empty-p (string-trim cron-str)))
-               (not (member from-str org-done-keywords))
-               (member to-str org-done-keywords))
-      (save-excursion
-        (org-back-to-heading t)
-        (let ((cron-arity (org-repeat-by-cron--cron-rule-arity cron-str)))
-          (if (not cron-arity)
-              (message "[Cron-Repeat] Invalid cron rule: %s" cron-str)
-            (let* ((deadline-prop (org-entry-get pom org-repeat-by-cron-deadline-prop))
-                   (process-deadline nil)
-                   (process-schedule nil)
-                   (keep-sched (memq org-repeat-by-cron--repeater-raw '(both schedule)))
-                   (keep-dead  (memq org-repeat-by-cron--repeater-raw '(both deadline))))
-              
-              (cond
-               ;; REPEAT_DEADLINE is t，only deadline
-               ((string= deadline-prop "t")
-                (setq process-deadline t))
-               ;; REPEAT_DEADLINE as cron ，deal with deadline and schedule
-               ((org-repeat-by-cron--cron-rule-arity deadline-prop)
-                (setq process-deadline t)
-                (setq process-schedule t))
-               ;; nil or other, only schedule
-               (t
-                (setq process-schedule t)))
-
-              (cl-labels ((calc-next-ts (cron-val anchor-prop time-type)
-                            "Internal func, calculate the next timestamp, update anchor,
- and return time-string"
-                            (let* ((anchor-str (org-entry-get pom anchor-prop))
-                                   (day-and-p (string= (org-entry-get pom org-repeat-by-cron-day-and-prop) "t"))
-                                   (now (current-time))
-                                   (current-ts-str (org-entry-get pom (if (eq time-type 'deadline) "DEADLINE" "SCHEDULED")))
-                                   (base-time (or (and anchor-str (org-time-string-to-time anchor-str))
-                                                  now))
-                                   (next (org-repeat-by-cron-next-time
-                                          (org-repeat-by-cron--normalize-cron-rule cron-val)
-                                          (if (time-less-p base-time now) now base-time)
-                                          day-and-p))
-                                   (c-arity (org-repeat-by-cron--cron-rule-arity cron-val))
-                                   (fmt (org-repeat-by-cron--reschedule-use-time-p anchor-str c-arity current-ts-str)))
-                              (if next
-                                  (let ((next-raw (format-time-string fmt next)))
-                                    (org-entry-put pom anchor-prop next-raw)
-                                    next-raw)
-                                (message "[Cron-Repeat] Cannot find valid time before %s"
-                                         (+ (nth 5 (decode-time)) org-repeat-by-cron-max-search-year))
-                                nil))))
-
-                ;; 2. Schedule
-                (when process-schedule
-                  (let ((res (calc-next-ts cron-str org-repeat-by-cron-anchor-prop 'schedule)))
-                    (when res
-                      (unless keep-sched (org-schedule '(4)))
-                      (org-schedule nil res)
-                      (message "[Cron-Repeat] SCHEDULED repeat to %s" res))))
-
-                ;; 3. Deadline
-                (when process-deadline
-                  (let* ((is-t (string= deadline-prop "t"))
-                         (d-cron (if is-t cron-str deadline-prop))
-                         (d-anchor-prop (if is-t org-repeat-by-cron-anchor-prop org-repeat-by-cron-deadline-anchor-prop))
-                         (res (calc-next-ts d-cron d-anchor-prop 'deadline)))
-                    (when res
-                      (unless keep-dead (org-deadline '(4)))
-                      (org-deadline nil res)
-                      (message "[Cron-Repeat] DEADLINE repeat to %s"  res))))))))))))
+The base time for the calculation is chosen in the following order
+of precedence: the anchor property's time, the existing SCHEDULED
+or DEADLINE time, and finally the current time.
 
+Upon a successful reschedule, the entry's TODO state is reset to
+the first non-done state (TODO by default)."
+  (when (org-entry-is-done-p)
+    (save-excursion
+      (org-back-to-heading t)
+      (let* ((day-and-str   (org-entry-get (point) org-repeat-by-cron-day-and-prop nil))
+             (day-and-p     (if (string= day-and-str "t") t nil))
+             (cron-str       (org-entry-get (point) org-repeat-by-cron-cron-prop nil))
+             (deadline-str (org-entry-get (point) org-repeat-by-cron-deadline-prop nil))
+             (resched-func (if (string= deadline-str "t") #'org-deadline #'org-schedule))
+             (has-cron       (and cron-str     (> (length (string-trim cron-str))     0)))
+             (repeat-time     (if (string= deadline-str "t")  (org-get-deadline-time (point)) (org-get-scheduled-time (point))))
+             (anchor-str     (org-entry-get (point) org-repeat-by-cron-anchor-prop nil))
+             (anchor-time    (when (and anchor-str (> (length (string-trim anchor-str)) 0))
+                               (org-time-string-to-time anchor-str)))
+             (base-time           (or anchor-time repeat-time (current-time)))
+             (cron-arity          (when has-cron     (org-repeat-by-cron--cron-rule-arity cron-str)))
+             (norm-cron      (when cron-arity        (org-repeat-by-cron--normalize-cron-rule cron-str)))
+             (resched-str  (org-entry-get (point) (if (string= deadline-str "t") "DEADLINE" "SCHEDULED")))
+             (fmt            (org-repeat-by-cron--reschedule-use-time-p
+                              anchor-str cron-arity resched-str)))
+        (when has-cron
+          (when (and has-cron (null cron-arity))
+              (message "[repeat] invalid cron rule, skipping"))
+          (when-let* ((bigger-time (if (time-less-p base-time nil) (current-time)
+                                     base-time))
+                      (next (org-repeat-by-cron-next-time norm-cron base-time day-and-p)))
+            (when next
+              (funcall resched-func nil (format-time-string fmt next))
+              ;; When using deadline, we need to manually clear SHCEDULED timestamp
+              (when (string= deadline-str "t") (org-schedule '(4)))
+              (org-entry-put (point) org-repeat-by-cron-anchor-prop
+                             (format-time-string fmt next))
+              (org-todo 'todo)
+              (message "[repeat] repeated to %s%s"
+                       (format-time-string fmt next)
+                       (if has-cron ", reset state")))))))))
 ;;;###autoload
 (define-minor-mode global-org-repeat-by-cron-mode
-  "Toggle global cron-style task repetition in Org mode.
-
-When enabled, tasks with a REPEAT_CRON property will automatically
-reschedule themselves to the next occurrence matching the cron
-expression when marked as DONE. This mode adds functions
-to `org-trigger-hook' and `org-after-todo-state-change-hook'."
+  "A global minor mode globally enable org-repeat-by-cron."
   :init-value nil
   :global t
   :group 'org-repeat-by-cron
   (if global-org-repeat-by-cron-mode
-      (progn
-        (add-hook 'org-trigger-hook #'org-repeat-by-cron-on-done)
-        (add-hook 'org-after-todo-state-change-hook #'org-repeat-by-cron--ensure-repeater))
-    (remove-hook 'org-trigger-hook #'org-repeat-by-cron-on-done)
-    (remove-hook 'org-after-todo-state-change-hook #'org-repeat-by-cron--ensure-repeater)))
+      (add-hook 'org-after-todo-state-change-hook #'org-repeat-by-cron-on-done)
+    (remove-hook 'org-after-todo-state-change-hook #'org-repeat-by-cron-on-done)))
 
 (provide 'org-repeat-by-cron)
+
 ;;; org-repeat-by-cron.el ends here
